#!/usr/bin/env python

"""quicksort.py

Sorts a list of elements in-place using the Quick-Sort algorithm.
Tuples or strings to be sorted must first be converted to lists
and can be converted back after the sort (see examples below).

Dinu C. Gherman
"""

import string

def qs(A, left, right):
	# Terminate if bounds cross.
	if left > right:
		return
	
	# Set initial running indices and 
	# determine Pivot element x, which
	# could also be like this: x = random.choice(A)
	i, j = left, right
	x = A[(left + right) / 2]
	
	done = 0
	while not done:
		# Skip well-ordered elements.
		while A[i] < x:
			i = i + 1
		while A[j] > x:
			j = j - 1

		# Swap elements.
		if i <= j:
			A[i], A[j] = A[j], A[i]
			i = i + 1
			j = j - 1

		# Stop criterion (overlapping bounds).
		if i > j:
			done = 1

	# Recursively sort the two sublists.
	qs(A, left, j)
	qs(A, i, right)


def quicksort(A, left=None, right=None):
	# Set left- and rightmost bounds if nothing specified.
	if left == right == None:
		left = 0
		right = len(A) - 1

	# Call the function doing the real work.
	qs(A, left, right)


if __name__ == '__main__':
	# Sample of a number sort.
	array = [5,4,1,2]
	print array
	quicksort(array)
	print array
	print

	# Sample of a character sort.
	array = "Aide moi o media"
	print array
	array = map(None, array)
	quicksort(array)
	print string.join(array, "")
	print
